@@ -39,7 +39,7 @@
 
 #include "window.h"
 #include "shared/helpers.h"
-#include "shared/matrix.h"
+#include <libweston/matrix.h>
 
 /* Our points for the calibration must be not be on a line */
 static const struct {
@@ -287,7 +287,7 @@ main(int argc, char *argv[])
 			break;
 		}
 	}
-	
+
 	display = display_create(&argc, argv);
 
 	if (display == NULL) {
