@@ -34,6 +34,7 @@
 #include <assert.h>
 #include <unistd.h>
 #include <time.h>
+#include <errno.h>
 
 #include <linux/input.h>
 #include <wayland-client.h>
@@ -328,7 +329,8 @@ main(int argc, char *argv[])
 
 	display = display_create(&argc, argv);
 	if (display == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
 		return -1;
 	}
 
