@@ -37,7 +37,7 @@
 #include <wayland-client.h>
 #include "window.h"
 #include "fullscreen-shell-unstable-v1-client-protocol.h"
-#include "shared/zalloc.h"
+#include <libweston/zalloc.h>
 
 struct fs_output {
 	struct wl_list link;
@@ -280,6 +280,9 @@ key_handler(struct window *window, struct input *input, uint32_t time,
 		break;
 
 	case XKB_KEY_z:
+		if (fullscreen->fullscreen)
+			break;
+
 		current_size = (current_size + 1) % 4;
 		fullscreen->width = widths[current_size];
 		fullscreen->height = heights[current_size];
