@@ -36,6 +36,7 @@
 #include <time.h>
 #include <cairo.h>
 #include <assert.h>
+#include <errno.h>
 #include <linux/input.h>
 
 #include <wayland-client.h>
@@ -419,7 +420,8 @@ main(int argc, char *argv[])
 
 	d = display_create(&argc, argv);
 	if (d == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
 		return -1;
 	}
 
