@@ -31,9 +31,9 @@
 #include <errno.h>
 #include <sys/socket.h>
 
-#include "compositor.h"
+#include <libweston/libweston.h>
 #include "compositor/weston.h"
-#include "xwayland/xwayland-api.h"
+#include <libweston/xwayland-api.h>
 #include "shared/helpers.h"
 
 struct wet_xwayland {
