@@ -62,9 +62,6 @@ struct exposay_output {
 	int num_surfaces;
 	int grid_size;
 	int surface_size;
-
-	int hpadding_outer;
-	int vpadding_outer;
 	int padding_inner;
 };
 
@@ -235,6 +232,11 @@ get_shell_surface(struct weston_surface *surface);
 struct workspace *
 get_current_workspace(struct desktop_shell *shell);
 
+void
+get_output_work_area(struct desktop_shell *shell,
+		     struct weston_output *output,
+		     pixman_rectangle32_t *area);
+
 void
 lower_fullscreen_layer(struct desktop_shell *shell,
 		       struct weston_output *lowering_output);
