@@ -28,7 +28,7 @@
 
 #include "config.h"
 
-#include "compositor.h"
+#include <libweston/libweston.h>
 
 struct weston_launcher;
 
