@@ -30,9 +30,10 @@
 #include <unistd.h>
 #include <sys/types.h>
 
-#include "compositor.h"
+#include <libweston/libweston.h>
 #include "linux-dmabuf.h"
 #include "linux-dmabuf-unstable-v1-server-protocol.h"
+#include "libweston-internal.h"
 
 static void
 linux_dmabuf_buffer_destroy(struct linux_dmabuf_buffer *buffer)
@@ -141,9 +142,6 @@ destroy_linux_dmabuf_wl_buffer(struct wl_resource *resource)
 	assert(buffer->buffer_resource == resource);
 	assert(!buffer->params_resource);
 
-	if (buffer->gem_handle_close_func)
-		buffer->gem_handle_close_func(buffer);
-
 	if (buffer->user_data_destroy_func)
 		buffer->user_data_destroy_func(buffer);
 
@@ -261,15 +259,18 @@ params_create_common(struct wl_client *client,
 		}
 	}
 
-	/* XXX: Some additional sanity checks could be done with respect
-	 * to the fourcc format. A centralized collection (kernel or
-	 * libdrm) would be useful to avoid code duplication for these
-	 * checks (e.g. drm_format_num_planes).
-	 */
+	if (buffer->direct_display) {
+		if (!weston_compositor_dmabuf_can_scanout(buffer->compositor,
+							  buffer))
+			goto err_failed;
+
+		goto avoid_gpu_import;
+	}
 
 	if (!weston_compositor_import_dmabuf(buffer->compositor, buffer))
 		goto err_failed;
 
+avoid_gpu_import:
 	buffer->buffer_resource = wl_resource_create(client,
 						     &wl_buffer_interface,
 						     1, buffer_id);
@@ -337,35 +338,12 @@ params_create_immed(struct wl_client *client,
 			     format, flags);
 }
 
-static void
-params_add_dtrc_meta(struct wl_client *client,
-		    struct wl_resource *params_resource,
-		    uint32_t rfc_chroma_offset,
-		    uint32_t rfc_luma_offset)
-{
-	struct linux_dmabuf_buffer *buffer;
-
-	buffer = wl_resource_get_user_data(params_resource);
-	if (!buffer) {
-		wl_resource_post_error(params_resource,
-			ZWP_LINUX_BUFFER_PARAMS_V1_ERROR_ALREADY_USED,
-			"params was already used to create a wl_buffer");
-		return;
-	}
-
-	assert(buffer->params_resource == params_resource);
-	assert(!buffer->buffer_resource);
-
-	buffer->attributes.dtrc_meta =  rfc_luma_offset | ((uint64_t)rfc_chroma_offset << 32);
-}
-
 static const struct zwp_linux_buffer_params_v1_interface
 zwp_linux_buffer_params_implementation = {
 	params_destroy,
 	params_add,
 	params_create,
-	params_create_immed,
-	params_add_dtrc_meta
+	params_create_immed
 };
 
 static void
@@ -399,6 +377,7 @@ linux_dmabuf_create_params(struct wl_client *client,
 		wl_resource_create(client,
 				   &zwp_linux_buffer_params_v1_interface,
 				   version, params_id);
+	buffer->direct_display = false;
 	if (!buffer->params_resource)
 		goto err_dealloc;
 
@@ -444,21 +423,6 @@ linux_dmabuf_buffer_get(struct wl_resource *resource)
 	return buffer;
 }
 
-/** Set drmbackend-private data
- *
- * set the drm gem handle close callback in the linux_dmabuf_buffer
- *
- * \param buffer The linux_dmabuf_buffer object to set for.
- * \param func Destructor function to be called to close gem handle
- *             when the linux_dmabuf_buffer gets destroyed.
- */
-WL_EXPORT void
-linux_dmabuf_buffer_gem_handle_close_cb(struct linux_dmabuf_buffer *buffer,
-				  dmabuf_gem_handle_close_func func)
-{
-	buffer->gem_handle_close_func = func;
-}
-
 /** Set renderer-private data
  *
  * Set the user data for the linux_dmabuf_buffer. It is invalid to overwrite
@@ -566,45 +530,6 @@ bind_linux_dmabuf(struct wl_client *client,
 			free(modifiers);
 	}
 	free(formats);
-	formats = NULL;
-
-	if (compositor->backend->query_dmabuf_formats && compositor->backend->query_dmabuf_modifiers) {
-		compositor->backend->query_dmabuf_formats(compositor, &formats, &num_formats);
-
-		if (!formats)
-			return;
-
-		for (i = 0; i < num_formats; i++) {
-			compositor->backend->query_dmabuf_modifiers(compositor,
-							     formats[i],
-							     &modifiers,
-							     &num_modifiers);
-
-			/* send DRM_FORMAT_MOD_INVALID token when no modifiers are supported
-			 * for this format */
-			if (num_modifiers == 0) {
-				num_modifiers = 1;
-				modifiers = &modifier_invalid;
-			}
-			for (j = 0; j < num_modifiers; j++) {
-				if (version >= ZWP_LINUX_DMABUF_V1_MODIFIER_SINCE_VERSION) {
-					uint32_t modifier_lo = modifiers[j] & 0xFFFFFFFF;
-					uint32_t modifier_hi = modifiers[j] >> 32;
-					zwp_linux_dmabuf_v1_send_modifier(resource,
-									  formats[i],
-									  modifier_hi,
-									  modifier_lo);
-				} else if (modifiers[j] == DRM_FORMAT_MOD_LINEAR ||
-					   modifiers == &modifier_invalid) {
-					zwp_linux_dmabuf_v1_send_format(resource,
-									formats[i]);
-				}
-			}
-			if (modifiers != &modifier_invalid)
-				free(modifiers);
-		}
-		free(formats);
-	}
 }
 
 /** Advertise linux_dmabuf support
