@@ -324,7 +324,7 @@ emit_vblank_timestamp(struct timeline_emit_context *ctx, void *obj)
 {
 	struct timespec *ts = obj;
 
-	fprintf(ctx->cur, "\"vblank\":[%" PRId64 ", %ld]",
+	fprintf(ctx->cur, "\"vblank_monotonic\":[%" PRId64 ", %ld]",
 		(int64_t)ts->tv_sec, ts->tv_nsec);
 
 	return 1;
