@@ -34,6 +34,18 @@
 
 #include "shared/os-compatibility.h"
 #include "weston-test-client-helper.h"
+#include "weston-test-fixture-compositor.h"
+
+static enum test_result_code
+fixture_setup(struct weston_test_harness *harness)
+{
+	struct compositor_setup setup;
+
+	compositor_setup_defaults(&setup);
+
+	return weston_test_harness_execute_as_client(harness, &setup);
+}
+DECLARE_FIXTURE_SETUP(fixture_setup);
 
 /* These three functions are copied from shared/os-compatibility.c in order to
  * behave like older clients, and allow ftruncate() to shrink the file’s size,
