@@ -27,6 +27,18 @@
 
 #include <string.h>
 #include "weston-test-client-helper.h"
+#include "weston-test-fixture-compositor.h"
+
+static enum test_result_code
+fixture_setup(struct weston_test_harness *harness)
+{
+	struct compositor_setup setup;
+
+	compositor_setup_defaults(&setup);
+
+	return weston_test_harness_execute_as_client(harness, &setup);
+}
+DECLARE_FIXTURE_SETUP(fixture_setup);
 
 /**
  * Test (un)plugging devices
@@ -249,17 +261,18 @@ TEST(get_device_after_destroy)
 	struct wl_touch *wl_touch;
 
 	/* There's a race:
-	 *  1) compositor destroyes device
-	 *  2) client asks for the device, because it has not got
-	 *     new capabilities yet
-	 *  3) compositor gets request with new_id for destroyed device
+	 *  1) compositor destroys device
+	 *  2) client asks for the device, because it has not received
+	 *     the new capabilities yet
+	 *  3) compositor gets the request with a new_id for the
+	 *     destroyed device
 	 *  4) client uses the new_id
 	 *  5) client gets new capabilities, destroying the objects
 	 *
-	 * If compositor just bail out in step 3) and does not create
-	 * resource, then client gets error in step 4) - even though
-	 * it followed the protocol (it just didn't know about new
-	 * capabilities).
+	 * If the compositor just bails out in step 3) and does not
+	 * create the resource, then the client gets an error in step 4)
+	 * - even though it followed the protocol (it just didn't know
+	 * about new capabilities).
 	 *
 	 * This test simulates this situation
 	 */
@@ -309,3 +322,35 @@ TEST(get_device_after_destroy_multiple)
 		get_device_after_destroy();
 	}
 }
+
+TEST(seats_have_names)
+{
+	struct client *cl = create_client_and_test_surface(100, 100, 100, 100);
+	struct input *input;
+
+	wl_list_for_each(input, &cl->inputs, link) {
+		assert(input->seat_name);
+	}
+}
+
+TEST(seat_destroy_and_recreate)
+{
+	struct client *cl = create_client_and_test_surface(100, 100, 100, 100);
+
+	weston_test_device_release(cl->test->weston_test, "seat");
+	/* Roundtrip to receive and handle the seat global removal event */
+	client_roundtrip(cl);
+
+	assert(!cl->input);
+
+	weston_test_device_add(cl->test->weston_test, "seat");
+	/* First roundtrip to send request and receive new seat global */
+	client_roundtrip(cl);
+	/* Second roundtrip to handle seat events and set up input devices */
+	client_roundtrip(cl);
+
+	assert(cl->input);
+	assert(cl->input->pointer);
+	assert(cl->input->keyboard);
+	assert(cl->input->touch);
+}
