@@ -30,6 +30,19 @@
 #include <assert.h>
 
 #include "weston-test-client-helper.h"
+#include "weston-test-fixture-compositor.h"
+
+static enum test_result_code
+fixture_setup(struct weston_test_harness *harness)
+{
+	struct compositor_setup setup;
+
+	compositor_setup_defaults(&setup);
+	setup.logging_scopes = "log,proto,test-harness-plugin";
+
+	return weston_test_harness_execute_as_client(harness, &setup);
+}
+DECLARE_FIXTURE_SETUP(fixture_setup);
 
 static struct wl_subcompositor *
 get_subcompositor(struct client *client)
