@@ -29,6 +29,18 @@
 #include <string.h>
 
 #include "weston-test-client-helper.h"
+#include "weston-test-fixture-compositor.h"
+
+static enum test_result_code
+fixture_setup(struct weston_test_harness *harness)
+{
+	struct compositor_setup setup;
+
+	compositor_setup_defaults(&setup);
+
+	return weston_test_harness_execute_as_client(harness, &setup);
+}
+DECLARE_FIXTURE_SETUP(fixture_setup);
 
 #define NUM_SUBSURFACES 3
 
@@ -694,11 +706,11 @@ destroy_permu_object(struct wl_surface **surfs,
 	int h = (i + 1) / 2;
 
 	if (i & 1) {
-		fprintf(stderr, " [sub  %2d]", h);
+		testlog(" [sub  %2d]", h);
 		wl_subsurface_destroy(subs[h]);
 		subs[h] = NULL;
 	} else {
-		fprintf(stderr, " [surf %2d]", h);
+		testlog(" [surf %2d]", h);
 		wl_surface_destroy(surfs[h]);
 		surfs[h] = NULL;
 	}
@@ -736,15 +748,15 @@ TEST(test_subsurface_destroy_permutations)
 
 		create_subsurface_tree(client, surfs, subs, test_size);
 
-		fprintf(stderr, "permu");
+		testlog("permu");
 
 		for (i = 0; i < NSTEPS; i++)
-			fprintf(stderr, " %2d", per.cnt[i]);
+			testlog(" %2d", per.cnt[i]);
 
 		for (i = 0; i < NSTEPS; i++)
 			destroy_permu_object(surfs, subs, per.cnt[i]);
 
-		fprintf(stderr, "\n");
+		testlog("\n");
 		client_roundtrip(client);
 
 		destroy_subsurface_tree(surfs, subs, test_size);
@@ -752,5 +764,5 @@ TEST(test_subsurface_destroy_permutations)
 	}
 
 	client_roundtrip(client);
-	fprintf(stderr, "tried %d destroy permutations\n", counter);
+	testlog("tried %d destroy permutations\n", counter);
 }
