@@ -30,11 +30,23 @@
 
 #include <libweston/libweston.h>
 #include "compositor/weston.h"
+#include "weston-test-runner.h"
+#include "weston-test-fixture-compositor.h"
 
-static void
-surface_transform(void *data)
+static enum test_result_code
+fixture_setup(struct weston_test_harness *harness)
 {
-	struct weston_compositor *compositor = data;
+	struct compositor_setup setup;
+
+	compositor_setup_defaults(&setup);
+
+	return weston_test_harness_execute_as_plugin(harness, &setup);
+}
+DECLARE_FIXTURE_SETUP(fixture_setup);
+
+PLUGIN_TEST(surface_transform)
+{
+	/* struct weston_compositor *compositor; */
 	struct weston_surface *surface;
 	struct weston_view *view;
 	float x, y;
@@ -56,19 +68,4 @@ surface_transform(void *data)
 	weston_view_update_transform(view);
 	weston_view_to_global_float(view, 50, 40, &x, &y);
 	assert(x == 200 && y == 340);
-
-	weston_compositor_exit(compositor);
-}
-
-WL_EXPORT int
-wet_module_init(struct weston_compositor *compositor,
-		int *argc, char *argv[])
-{
-	struct wl_event_loop *loop;
-
-	loop = wl_display_get_event_loop(compositor->wl_display);
-
-	wl_event_loop_add_idle(loop, surface_transform, compositor);
-
-	return 0;
 }
