@@ -31,11 +31,13 @@
 #include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <time.h>
 #include <pixman.h>
 
 #include <wayland-client-protocol.h>
 #include "weston-test-runner.h"
 #include "weston-test-client-protocol.h"
+#include "viewporter-client-protocol.h"
 
 struct client {
 	struct wl_display *wl_display;
@@ -56,6 +58,7 @@ struct client {
 	int has_argb;
 	struct wl_list global_list;
 	bool has_wl_drm;
+	struct wl_list output_list; /* struct output::link */
 };
 
 struct global {
@@ -74,6 +77,8 @@ struct test {
 };
 
 struct input {
+	struct client *client;
+	uint32_t global_name;
 	struct wl_seat *wl_seat;
 	struct pointer *pointer;
 	struct keyboard *keyboard;
@@ -90,6 +95,17 @@ struct pointer {
 	int y;
 	uint32_t button;
 	uint32_t state;
+	uint32_t axis;
+	double axis_value;
+	uint32_t motion_time_msec;
+	uint32_t button_time_msec;
+	uint32_t axis_time_msec;
+	uint32_t axis_stop_time_msec;
+	struct timespec input_timestamp;
+	struct timespec motion_time_timespec;
+	struct timespec button_time_timespec;
+	struct timespec axis_time_timespec;
+	struct timespec axis_stop_time_timespec;
 };
 
 struct keyboard {
@@ -105,6 +121,9 @@ struct keyboard {
 		int rate;
 		int delay;
 	} repeat_info;
+	uint32_t key_time_msec;
+	struct timespec input_timestamp;
+	struct timespec key_time_timespec;
 };
 
 struct touch {
@@ -117,10 +136,18 @@ struct touch {
 	int up_id; /* id of last wl_touch.up event */
 	int frame_no;
 	int cancel_no;
+	uint32_t down_time_msec;
+	uint32_t up_time_msec;
+	uint32_t motion_time_msec;
+	struct timespec input_timestamp;
+	struct timespec down_time_timespec;
+	struct timespec up_time_timespec;
+	struct timespec motion_time_timespec;
 };
 
 struct output {
 	struct wl_output *wl_output;
+	struct wl_list link; /* struct client::output_list */
 	int x;
 	int y;
 	int width;
@@ -137,7 +164,7 @@ struct buffer {
 
 struct surface {
 	struct wl_surface *wl_surface;
-	struct output *output;
+	struct output *output; /* not owned */
 	int x;
 	int y;
 	int width;
@@ -152,9 +179,23 @@ struct rectangle {
 	int height;
 };
 
+struct range {
+	int a;
+	int b;
+};
+
 struct client *
 create_client(void);
 
+void
+client_destroy(struct client *client);
+
+struct surface *
+create_test_surface(struct client *client);
+
+void
+surface_destroy(struct surface *surface);
+
 struct client *
 create_client_and_test_surface(int x, int y, int width, int height);
 
@@ -195,13 +236,18 @@ screenshot_output_filename(const char *basename, uint32_t seq);
 char *
 screenshot_reference_filename(const char *basename, uint32_t seq);
 
+char *
+image_filename(const char *basename);
+
 bool
 check_images_match(pixman_image_t *img_a, pixman_image_t *img_b,
-		   const struct rectangle *clip);
+		   const struct rectangle *clip,
+		   const struct range *prec);
 
 pixman_image_t *
 visualize_image_difference(pixman_image_t *img_a, pixman_image_t *img_b,
-			   const struct rectangle *clip_rect);
+			   const struct rectangle *clip_rect,
+			   const struct range *prec);
 
 bool
 write_image_as_png(pixman_image_t *image, const char *fname);
@@ -212,4 +258,30 @@ load_image_from_png(const char *fname);
 struct buffer *
 capture_screenshot_of_output(struct client *client);
 
+bool
+verify_screen_content(struct client *client,
+		      const char *ref_image,
+		      int ref_seq_no,
+		      const struct rectangle *clip,
+		      int seq_no);
+
+struct buffer *
+client_buffer_from_image_file(struct client *client,
+			      const char *basename,
+			      int scale);
+
+void *
+bind_to_singleton_global(struct client *client,
+			 const struct wl_interface *iface,
+			 int version);
+
+struct wp_viewport *
+client_create_viewport(struct client *client);
+
+void
+fill_image_with_color(pixman_image_t *image, pixman_color_t *color);
+
+pixman_color_t *
+color_rgb888(pixman_color_t *tmp, uint8_t r, uint8_t g, uint8_t b);
+
 #endif
