@@ -230,7 +230,7 @@ weston_xserver_destroy(struct wl_listener *l, void *data)
 	if (wxs->loop)
 		weston_xserver_shutdown(wxs);
 
-	weston_compositor_log_scope_destroy(wxs->wm_debug);
+	weston_log_scope_destroy(wxs->wm_debug);
 
 	free(wxs);
 }
@@ -396,8 +396,7 @@ weston_module_init(struct weston_compositor *compositor)
 	}
 
 	wxs->wm_debug =
-		weston_compositor_add_log_scope(wxs->compositor->weston_log_ctx,
-						"xwm-wm-x11",
+		weston_compositor_add_log_scope(wxs->compositor, "xwm-wm-x11",
 						"XWM's window management X11 events\n",
 						NULL, NULL, NULL);
 
